import React from "react";
import { connect } from "react-redux";
import { compose } from "recompose";
import memoize from "memoize-one";

import { Segment } from "semantic-ui-react";
import uuidv1 from "uuid/v1";

import { withFirebase } from "../Firebase";

import * as LOT_ACTIONS from "../../actions/lot";
import { STATUS, REFRESH } from "../../constants/lot";

import * as ROUTES from "../../constants/routes";

import * as FORMAT from "../../utils/Formater";
import * as BIT from "../../utils/Bitwise";

import "./Lots.scss";

class UserLotsList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      top5: true
    };
  }

  switchTop5() {
    const { top5 } = this.state;
    this.setState({ top5: !top5 });
  }

  listObjectToArray(obj) {
    let arr = [];
    if (obj instanceof Array) arr = [...obj];
    else if (obj instanceof Object) {
      arr = obj.toArray((val, uid) => ({ ...val, uid }));
    }
    return arr;
  }

  filter = memoize((list, top5) => {
    const arr = list instanceof Array ? list : JSON.parse(list);
    if (top5) return arr.slice(0, 5);
    return arr;
    // list.filter(item => item.text.includes(filterText))
  });

  header(top5) {
    return (
      <div className="lots-list_header flex">
        <div className="lots-list_title grey">{`${
          top5 ? "Top 5 user lots" : "User lots"
        }`}</div>
        <div
          className="lots-list_switcher blue"
          onClick={this.switchTop5.bind(this)}
        >{`${top5 ? "View all" : "View top 5"}`}</div>
      </div>
    );
  }

  componentDidMount() {
    this.fetchData();
  }

  componentDidUpdate() {
    if (BIT.isset(this.props.needRefresh, REFRESH.LIST)) {
      console.log("NEED REFRESH LIST");
      this.fetchData();
    }
  }

  fetchData() {
    const {
      firebase,
      fetchLotsListStart,
      fetchLotsListSuccess,
      fetchLotsListError
    } = this.props;

    fetchLotsListStart();

    firebase.lotsByUser().then(response => {
      try {
        const lots = JSON.parse(response.data.lots);
        fetchLotsListSuccess(lots);
      } catch (err) {
        fetchLotsListError(err);
      }
    });
  }

  render() {
    const { lots, loading /*error*/ } = this.props;
    const { top5 } = this.state;

    const len = lots.length;
    const filtered = this.filter(lots, top5);

    return (
      <Segment className="lots-list" loading={loading}>
        {(show => (show ? this.header(top5) : null))(len > 5)}
        {filtered.map(lot => (
          <LotCard lot={lot} key={uuidv1()} />
        ))}
      </Segment>
    );
  }
}

const mapStateToProps = state => {
  const {
    userLots,
    userLotsFetching,
    userLotsFetchingError,
    needRefresh
  } = state.lot;

  return {
    lots: userLots,
    loading: userLotsFetching,
    error: userLotsFetchingError,
    needRefresh
  };
};
const mapDispatchToProps = dispatch => ({
  fetchLotsListStart: () => dispatch(LOT_ACTIONS.fetchLotsListStart()),
  fetchLotsListSuccess: data =>
    dispatch(LOT_ACTIONS.fetchLotsListSuccess(data)),
  fetchLotsListError: error => dispatch(LOT_ACTIONS.fetchLotsListError(error))
});

export default compose(
  withFirebase,
  connect(
    mapStateToProps,
    mapDispatchToProps
  )
)(UserLotsList);

export const LotCard = ({
  lot,
  link = true,
  className = "",
  title = false
}) => (
  <div className={`lot ${className}`}>
    {title && <div className="lot_title lot_cell">{lot.title}</div>}
    <div className="lot_card-row">
      <div className="lot_address lot_cell">
        {FORMAT.addressSpan({
          address: lot.route.address,
          href: link ? ROUTES.LOT.replace(/:[^:]+$/, lot.uid) : null
        })}
      </div>
      <div className="lot_price lot_cell ">
        <div
          className={
            lot.status === STATUS.FOR_SALE && lot.userPrice ? "red" : "grey"
          }
        >
          {FORMAT.money(
            (lot.status === STATUS.FOR_SALE && lot.userPrice) || lot.price
          )}
        </div>
        <div className="lot_price_change blue small">+ 3%</div>
      </div>
    </div>
    <div className="lot_card-row">
      <div className="lot_position lot_cell">
        {FORMAT.geoCoords(lot.position)}
      </div>
    </div>
  </div>
);
