import React, { Fragment } from "react";
import { connect } from "react-redux";
import { withFirebase } from "../Firebase";
import { compose } from "recompose";
import {
  Table,
  Popup,
  Pagination,
  Icon,
  Form,
  Message
} from "semantic-ui-react";
import uuidv1 from "uuid/v1";

import { TableRowLoader } from "../Loaders";

import * as FORMAT from "../../utils/Formater";

const DEFAULT_STATE = {
  payments: [],
  pageData: [],
  loading: false,
  error: null,
  activePage: 1,
  totalPages: 1,
  total: 0,
  pageSize: parseInt(localStorage.getItem("avvy:paymentListPageSize") || 10, 10)
};

class UserPaymentsList extends React.Component {
  constructor(props) {
    super(props);
    this.state = DEFAULT_STATE;
  }

  getData(attempt = 0) {
    const { authUser, firebase } = this.props;
    const { loading, pageSize, payments } = this.state;
    this.setState({ error: null });

    if (
      firebase.auth.currentUser &&
      loading === false &&
      payments.length === 0
    ) {
      this.setState({ ...DEFAULT_STATE, loading: true });

      firebase.auth.currentUser
        .getIdToken(/* forceRefresh */ true)
        .then(token => {
          return fetch(
            `${process.env.REACT_APP_API_SERVER}/api/v1/user/${
              authUser.uid
            }/payments`,
            {
              mode: "cors",
              method: "get",
              headers: {
                Accept: "application/json",
                Authorization: `Bearer ${token}`
              }
            }
          );
        })
        .then(response => {
          if (response.status === 200) return response.json();
          else
            this.setState({
              ...DEFAULT_STATE,
              error: `FETCH PAYMENTS DATA ERROR: response status ${
                response.status
              }`
            });
        })
        .then(json => {
          const data = json.operations;
          const total = data.length;
          const payments = data
            .map(record => ({
              ...record,
              time:
                record.time === "0001-01-01T00:00:00Z"
                  ? null
                  : new Date(record.time).getTime()
            }))
            .sort((a, b) => b.time - a.time)
            .map((record, i) => ({ ...record, num: i + 1 }));

          const totalPages = Math.ceil(total / pageSize);
          const pageData = payments.slice(0, pageSize);

          this.setState({
            payments,
            pageData,
            total,
            totalPages,
            activePage: 1,
            loading: false,
            error: null
          });
        })
        .catch(error => {
          this.setState({
            ...DEFAULT_STATE,
            error: `FETCH PAYMENTS DATA ERROR: response status ${
              error.toString ? error.toString() : error
            }`
          });
        });
    } else if (attempt < 100) {
      setTimeout(this.getData.bind(this), 100, 1 + parseInt(attempt, 10));
    } else {
      this.setState({ ...DEFAULT_STATE, error: "Timed out" });
    }
  }

  componentDidMount() {
    this.getData();
  }

  getPageData(page, size = this.state.pageSize) {
    const { payments } = this.state;
    const totalPages = Math.ceil(payments.length / size);
    const start = (page - 1) * size;
    const end = start + size;
    const pageData = payments.slice(start, end);
    this.setState({ pageData, totalPages, activePage: page, pageSize: size });
  }

  handlePageChange(e, { activePage }) {
    this.getPageData(activePage);
  }

  handlePageSizeChange(e) {
    const newPageSize = parseInt(e.currentTarget.value, 10);
    localStorage.setItem("avvy:paymentListPageSize", newPageSize);
    this.getPageData(1, newPageSize);
  }

  render() {
    const {
      pageData,
      loading,
      activePage,
      totalPages,
      total,
      pageSize,
      error
    } = this.state;

    return (
      <Fragment>
        <h3>Payments</h3>
        <div
          style={{
            display: "flex",
            alignItems: "center"
          }}
        >
          <Pagination
            activePage={activePage}
            totalPages={totalPages}
            onPageChange={this.handlePageChange.bind(this)}
            ellipsisItem={{
              content: <Icon name="ellipsis horizontal" />,
              icon: true
            }}
            firstItem={{
              content: <Icon name="angle double left" />,
              icon: true
            }}
            lastItem={{
              content: <Icon name="angle double right" />,
              icon: true
            }}
            prevItem={{ content: <Icon name="angle left" />, icon: true }}
            nextItem={{ content: <Icon name="angle right" />, icon: true }}
          />
          <div style={{ marginLeft: 20 }}>
            {(() => {
              const from = (activePage - 1) * pageSize + 1;
              const to = Math.min(activePage * pageSize, total);
              const range = from === to ? " " + from : `s ${from}-${to}`;
              return `Opration${range} from ${total}`;
            })()}
          </div>
          <div style={{ flex: 1, marginLeft: 20, textAlign: "right" }}>
            <Form>
              <Form.Field
                inline
                label="Records per page"
                control="select"
                onChange={this.handlePageSizeChange.bind(this)}
                value={pageSize}
              >
                {[10, 20, 50].map(v => {
                  return (
                    <option key={uuidv1()} value={v}>
                      {v}
                    </option>
                  );
                })}
              </Form.Field>
            </Form>
          </div>
        </div>
        <Table compact>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell collapse="true">#</Table.HeaderCell>
              <Table.HeaderCell collapse="true">Operation ID</Table.HeaderCell>
              <Table.HeaderCell collapse="true">Payment Type</Table.HeaderCell>
              <Table.HeaderCell collapse="true">Amount</Table.HeaderCell>
              <Table.HeaderCell collapse="true">Time</Table.HeaderCell>
              <Table.HeaderCell>Description</Table.HeaderCell>
            </Table.Row>
          </Table.Header>
          <Table.Body>
            {loading && <TableRowLoader />}
            {error && (
              <Table.Row>
                <Table.Cell colSpan="100">
                  <Message negative>
                    <Message.Header>Failed to load data</Message.Header>
                    <p>{error}</p>
                  </Message>
                </Table.Cell>
              </Table.Row>
            )}
            {!loading && <PaymentsDataRows data={pageData} />}
          </Table.Body>
        </Table>
      </Fragment>
    );
  }
}

const mapStateToProps = state => {
  const { authUser, token } = state.session;
  return { authUser, token };
};

export default compose(
  withFirebase,
  connect(mapStateToProps)
)(UserPaymentsList);

const PaymentsDataRows = ({ data = [] }) =>
  data.map(r => <PaymentDataRow key={uuidv1()} data={r} />);

const PaymentDataRow = ({ data = {} }) => {
  const {
    num,
    doc_id = "",
    type = "",
    amount = 0,
    currency,
    time,
    description
  } = data;

  return (
    <Table.Row>
      <Table.Cell>{num}</Table.Cell>
      <Table.Cell>{doc_id}</Table.Cell>
      <Table.Cell>{type}</Table.Cell>
      <Table.Cell className={amount < 0 ? "red" : "green"} textAlign="right">
        {FORMAT.money(amount, currency)}
      </Table.Cell>
      <Table.Cell>
        {time && (
          <Popup
            content={FORMAT.date(time)}
            position="right center"
            on="click"
            trigger={<span>{FORMAT.date(time, true)}</span>}
          />
        )}
      </Table.Cell>
      <Table.Cell>{description}</Table.Cell>
    </Table.Row>
  );
};
